﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

using System.ComponentModel;

namespace DamageCalc
{
    public class DamageViewModel : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        private float exchangeRate = 1.65f;
        private float taxRate = 1.15f;

        private float latestPrice = 0;

        private bool priceChangeActive = false;

        private float twoDecimalPlaces(float input)
        {
            return (int)((input * 100) + 0.5f) / 100.0f;
        }

        public float Price
        {
            get
            {
                return latestPrice;
            }

            set
            {
                if (priceChangeActive) return;
                priceChangeActive = true;
                latestPrice = value;
                Damage = twoDecimalPlaces((value / exchangeRate) * taxRate);
                if (PropertyChanged != null)
                {
                    PropertyChanged(this, new PropertyChangedEventArgs("Damage"));
                }
                priceChangeActive = false;
            }
        }

        private float latestDamage;

        private bool damageChangeActive;

        public float Damage
        {
            get
            {
                return latestDamage;
            }
            set
            {
                if (damageChangeActive) return;
                damageChangeActive = true;

                latestDamage = value;
                Price = twoDecimalPlaces((latestDamage * (1 / taxRate)) * exchangeRate);

                if (PropertyChanged != null)
                {
                    PropertyChanged(this, new PropertyChangedEventArgs("Price"));
                }
                damageChangeActive = false;
            }
        }
    }
}
