﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using Microsoft.Phone.Controls;
using System.Data.Services.Client;
using ODataSample.NorthwindServiceReference;

namespace ODataSample
{
    public partial class MainPage : PhoneApplicationPage
    {
        NorthwindEntities context;
        private DataServiceCollection<Customer> customers;

        // Constructor
        public MainPage()
        {
            InitializeComponent();
        }
        
        private void MainPage_Loaded(object sender, RoutedEventArgs e)
        {
          //  App.ViewModel.Customers.Clear();
            context = new NorthwindEntities(new Uri("http://localhost:8321/NorthwindDataService.svc/"));
            // Initialize the context and the binding collection 
            customers = new DataServiceCollection<Customer>(context);

            // Define a LINQ query that returns all customers.
            var query = from cust in context.Customers
                        select cust;

            // Register for the LoadCompleted event.
            customers.LoadCompleted
                += new EventHandler<LoadCompletedEventArgs>(customers_LoadCompleted);

            // Load the customers feed by executing the LINQ query.
            customers.LoadAsync(query);

        }

        void customers_LoadCompleted(object sender, LoadCompletedEventArgs e)
        {
            if (e.Error == null)
            {
                // Handling for a paged data feed.
                if (customers.Continuation != null)
                {
                   
                    // Automatically load the next page.
                    customers.LoadNextPartialSetAsync();
                }
                else
                {
                    foreach (Customer c in customers)
                    {
                        //Add each customer to our View Model collection
                        App.ViewModel.Customers.Add(new CustomerViewModel(){SelectedCustomer = c});

                    }
                    // Set the data context of the list box control to the sample data.
                    this.LayoutRoot.DataContext = customers;
                }
            }
            else
            {
                MessageBox.Show(string.Format("An error has occurred: {0}", e.Error.Message));
            }
        }

        private void MainListBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            // If selected index is -1 (no selection) do nothing
            if (MainListBox.SelectedIndex == -1)
                return;

            // Navigate to the new page
            NavigationService.Navigate(new Uri("/DetailsPage.xaml?selectedItem=" + MainListBox.SelectedIndex, UriKind.Relative));

            // Reset selected index to -1 (no selection)
            MainListBox.SelectedIndex = -1;
        }
        
    }
}