﻿using System;
using System.Windows;
using Microsoft.Phone.Controls;
using System.IO;
using EugeneDotnetWP7Serialization.XML;
using EugeneDotnetWP7Serialization.DataContract;
using EugeneDotnetWP7Serialization.DataContractJSON;
using EugeneDotnetWP7Serialization.JSONNET;
using EugeneDotnetWP7Serialization.SharpSerialization;
using System.Diagnostics;
using System.IO.IsolatedStorage;
using SerializationTester.ViewModels;
using System.Collections.ObjectModel;

namespace SerializationTester
{
    public partial class MainPage : PhoneApplicationPage
    {
        Stopwatch sw = new Stopwatch();

        // Constructor
        public MainPage()
        {
            InitializeComponent();
        }

        private void GenerateDataMenuItem_Click(object sender, EventArgs e)
        {
            // navigate
            this.NavigationService.Navigate(new Uri("/GeneratorPopup.xaml", UriKind.Relative));
        }

        private void ClearMenuItem_Click(object sender, EventArgs e)
        {
            // navigate
            //this.NavigationService.Navigate(new Uri("/ClearPopup.xaml", UriKind.Relative));
        }

        private double TestMethod(Action method)
        {
            sw.Reset();
            sw.Start();
            method.Invoke();
            sw.Stop();
            return sw.ElapsedMilliseconds;
        }

        private void btnXMLSerialization_Click(object sender, RoutedEventArgs e)
        {
            IsolatedStorageFile myStore = IsolatedStorageFile.GetUserStoreForApplication();

            // time in milliseconds
            double serTime = TestMethod(() =>
                {
                    using (IsolatedStorageFileStream fs = new IsolatedStorageFileStream(
                        "XmlserializerContacts.store", FileMode.Create, myStore))
                    {
                        XMLSerializerHelper.Serialize(fs, ((App)App.Current).Contacts);
                    };
                    using (IsolatedStorageFileStream fs1 = new IsolatedStorageFileStream(
                        "XmlserializerLeads.store", FileMode.Create, myStore))
                    {
                        XMLSerializerHelper.Serialize(fs1, ((App)App.Current).Leads);
                    };
                    using (IsolatedStorageFileStream fs2 = new IsolatedStorageFileStream(
                        "XmlserializerActivities.store", FileMode.Create, myStore))
                    {
                        XMLSerializerHelper.Serialize(fs2, ((App)App.Current).Activities);
                    };                        
                }
                );

            Debug.WriteLine("Serialization: " + serTime.ToString() + "ms");

            double deSerTime = TestMethod(() =>
                {
                    using (IsolatedStorageFileStream fs = new IsolatedStorageFileStream(
                        "XmlserializerContacts.store", FileMode.Open, myStore))
                    {
                        var contacts = XMLSerializerHelper.Deserialize(fs, typeof(ObservableCollection<Contact>));
                    };
                    using (IsolatedStorageFileStream fs1 = new IsolatedStorageFileStream(
                        "XmlserializerLeads.store", FileMode.Open, myStore))
                    {
                        var leads = XMLSerializerHelper.Deserialize(fs1, typeof(ObservableCollection<Lead>));
                    };
                    using (IsolatedStorageFileStream fs2 = new IsolatedStorageFileStream(
                        "XmlserializerActivities.store", FileMode.Open, myStore))
                    {
                        var activities = XMLSerializerHelper.Deserialize(fs2, typeof(ObservableCollection<Activity>));
                    };                        
                }
            );

            Debug.WriteLine("Deserialization: " + deSerTime.ToString() + "ms");

            txtXmlSer.Text = serTime.ToString();
            txtXmlDeser.Text = deSerTime.ToString();
        }

        private void btnDataContractSerialization_Click(object sender, RoutedEventArgs e)
        {
            IsolatedStorageFile myStore = IsolatedStorageFile.GetUserStoreForApplication();

            // time in milliseconds
            double serTime = TestMethod(() =>
            {
                using (IsolatedStorageFileStream fs = new IsolatedStorageFileStream(
                    "DataCserializerContacts.store", FileMode.Create, myStore))
                {
                    DataContractSerializationHelper.Serialize(fs, ((App)App.Current).Contacts);
                };
                using (IsolatedStorageFileStream fs1 = new IsolatedStorageFileStream(
                    "DataCserializerLeads.store", FileMode.Create, myStore))
                {
                    DataContractSerializationHelper.Serialize(fs1, ((App)App.Current).Leads);
                };
                using (IsolatedStorageFileStream fs2 = new IsolatedStorageFileStream(
                    "DataCserializerActivities.store", FileMode.Create, myStore))
                {
                    DataContractSerializationHelper.Serialize(fs2, ((App)App.Current).Activities);
                };
            }
                );

            Debug.WriteLine("Serialization: " + serTime.ToString() + "ms");

            double deSerTime = TestMethod(() =>
            {
                using (IsolatedStorageFileStream fs = new IsolatedStorageFileStream(
                    "DataCserializerContacts.store", FileMode.Open, myStore))
                {
                    var contacts = DataContractSerializationHelper.Deserialize(fs, typeof(ObservableCollection<Contact>));
                };
                using (IsolatedStorageFileStream fs1 = new IsolatedStorageFileStream(
                    "DataCserializerLeads.store", FileMode.Open, myStore))
                {
                    var leads = DataContractSerializationHelper.Deserialize(fs1, typeof(ObservableCollection<Lead>));
                };
                using (IsolatedStorageFileStream fs2 = new IsolatedStorageFileStream(
                    "DataCserializerActivities.store", FileMode.Open, myStore))
                {
                    var activities = DataContractSerializationHelper.Deserialize(fs2, typeof(ObservableCollection<Activity>));
                };
            }
            );

            Debug.WriteLine("Deserialization: " + deSerTime.ToString() + "ms");

            txtDataCSer.Text = serTime.ToString();
            txtDataCDeser.Text = deSerTime.ToString();
        }

        private void btnDataContractJSONSerialization_Click(object sender, RoutedEventArgs e)
        {
            IsolatedStorageFile myStore = IsolatedStorageFile.GetUserStoreForApplication();

            // time in milliseconds
            double serTime = TestMethod(() =>
            {
                using (IsolatedStorageFileStream fs = new IsolatedStorageFileStream(
                    "DataCJSONserializerContacts.store", FileMode.Create, myStore))
                {
                    DataContractJSONSerializationHelper.Serialize(fs, ((App)App.Current).Contacts);
                };
                using (IsolatedStorageFileStream fs1 = new IsolatedStorageFileStream(
                    "DataCJSONserializerLeads.store", FileMode.Create, myStore))
                {
                    DataContractJSONSerializationHelper.Serialize(fs1, ((App)App.Current).Leads);
                };
                using (IsolatedStorageFileStream fs2 = new IsolatedStorageFileStream(
                    "DataCJSONserializerActivities.store", FileMode.Create, myStore))
                {
                    DataContractJSONSerializationHelper.Serialize(fs2, ((App)App.Current).Activities);
                };
            }
                );

            Debug.WriteLine("Serialization: " + serTime.ToString() + "ms");

            double deSerTime = TestMethod(() =>
            {
                using (IsolatedStorageFileStream fs = new IsolatedStorageFileStream(
                    "DataCJSONserializerContacts.store", FileMode.Open, myStore))
                {
                    var contacts = DataContractJSONSerializationHelper.Deserialize(fs, typeof(ObservableCollection<Contact>));
                };
                using (IsolatedStorageFileStream fs1 = new IsolatedStorageFileStream(
                    "DataCJSONserializerLeads.store", FileMode.Open, myStore))
                {
                    var leads = DataContractJSONSerializationHelper.Deserialize(fs1, typeof(ObservableCollection<Lead>));
                };
                using (IsolatedStorageFileStream fs2 = new IsolatedStorageFileStream(
                    "DataCJSONserializerActivities.store", FileMode.Open, myStore))
                {
                    var activities = DataContractJSONSerializationHelper.Deserialize(fs2, typeof(ObservableCollection<Activity>));
                };
            }
            );

            Debug.WriteLine("Deserialization: " + deSerTime.ToString() + "ms");

            txtDataCJSONSer.Text = serTime.ToString();
            txtDataCJSONDeser.Text = deSerTime.ToString();
        }

        private void btnJSONNETSerialization_Click(object sender, RoutedEventArgs e)
        {
            IsolatedStorageFile myStore = IsolatedStorageFile.GetUserStoreForApplication();

            // time in milliseconds
            double serTime = TestMethod(() =>
            {
                using (IsolatedStorageFileStream fs = new IsolatedStorageFileStream(
                    "JSONNETserializerContacts.store", FileMode.Create, myStore))
                {
                    JSONNETSerializationHelper.Serialize(fs, ((App)App.Current).Contacts);
                };
                using (IsolatedStorageFileStream fs1 = new IsolatedStorageFileStream(
                    "JSONNETserializerLeads.store", FileMode.Create, myStore))
                {
                    JSONNETSerializationHelper.Serialize(fs1, ((App)App.Current).Leads);
                };
                using (IsolatedStorageFileStream fs2 = new IsolatedStorageFileStream(
                    "JSONNETserializerActivities.store", FileMode.Create, myStore))
                {
                    JSONNETSerializationHelper.Serialize(fs2, ((App)App.Current).Activities);
                };
            }
                );

            Debug.WriteLine("Serialization: " + serTime.ToString() + "ms");

            double deSerTime = TestMethod(() =>
            {
                using (IsolatedStorageFileStream fs = new IsolatedStorageFileStream(
                    "JSONNETserializerContacts.store", FileMode.Open, myStore))
                {
                    var contacts = JSONNETSerializationHelper.Deserialize(fs, typeof(ObservableCollection<Contact>));
                };
                using (IsolatedStorageFileStream fs1 = new IsolatedStorageFileStream(
                    "JSONNETserializerLeads.store", FileMode.Open, myStore))
                {
                    var leads = JSONNETSerializationHelper.Deserialize(fs1, typeof(ObservableCollection<Lead>));
                };
                using (IsolatedStorageFileStream fs2 = new IsolatedStorageFileStream(
                    "JSONNETserializerActivities.store", FileMode.Open, myStore))
                {
                    var activities = JSONNETSerializationHelper.Deserialize(fs2, typeof(ObservableCollection<Activity>));
                };
            }
            );

            Debug.WriteLine("Deserialization: " + deSerTime.ToString() + "ms");

            txtJSONNETSer.Text = serTime.ToString();
            txtJSONNETDeser.Text = deSerTime.ToString();
        }

        private void btnSharpSerialization_Click(object sender, RoutedEventArgs e)
        {
            IsolatedStorageFile myStore = IsolatedStorageFile.GetUserStoreForApplication();

            // time in milliseconds
            double serTime = TestMethod(() =>
            {
                using (IsolatedStorageFileStream fs = new IsolatedStorageFileStream(
                    "SharpserializerContacts.store", FileMode.Create, myStore))
                {
                   SharpSerializerHelper.Serialize(fs, ((App)App.Current).Contacts);
                };
                using (IsolatedStorageFileStream fs1 = new IsolatedStorageFileStream(
                    "SharpserializerLeads.store", FileMode.Create, myStore))
                {
                    SharpSerializerHelper.Serialize(fs1, ((App)App.Current).Leads);
                };
                using (IsolatedStorageFileStream fs2 = new IsolatedStorageFileStream(
                    "SharpserializerActivities.store", FileMode.Create, myStore))
                {
                    SharpSerializerHelper.Serialize(fs2, ((App)App.Current).Activities);
                };
            }
                );

            Debug.WriteLine("Serialization: " + serTime.ToString() + "ms");

            double deSerTime = TestMethod(() =>
            {
                using (IsolatedStorageFileStream fs = new IsolatedStorageFileStream(
                    "SharpserializerContacts.store", FileMode.Open, myStore))
                {
                    var contacts = SharpSerializerHelper.Deserialize(fs) as ObservableCollection<Contact>;
                };
                using (IsolatedStorageFileStream fs1 = new IsolatedStorageFileStream(
                    "SharpserializerLeads.store", FileMode.Open, myStore))
                {
                    var leads = SharpSerializerHelper.Deserialize(fs1) as ObservableCollection<Lead>;
                };
                using (IsolatedStorageFileStream fs2 = new IsolatedStorageFileStream(
                    "SharpserializerActivities.store", FileMode.Open, myStore))
                {
                    var activities = SharpSerializerHelper.Deserialize(fs2) as ObservableCollection<Activity>;
                };
            }
            );

            Debug.WriteLine("Deserialization: " + deSerTime.ToString() + "ms");

            txtSharpSer.Text = serTime.ToString();
            txtSharpDeser.Text = deSerTime.ToString();
        }
    }
}
