﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using Microsoft.Phone.Controls;
using System.Collections.ObjectModel;

namespace CRMapp
{
    public partial class ContactPivotPage : PhoneApplicationPage
    {
        public ContactPivotPage()
        {
            InitializeComponent();
        }

        protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);

            // For tombstoning, save the oid of a particular record in the State object on OnNavigatedFrom
            // Restore it in OnNavigatedTo and use the Database.Select<T> method to extract the correct one from the database
            if (State.ContainsKey("DataContextObjectID"))
            {
                long cid = (long)State["DataContextObjectID"];
                this.DataContext = GetContact(cid);
                State.Remove("DataContextObjectID");
            }

            // Have we come here by navigating from the MainPage after selecting a record in the main list?
            if (this.NavigationContext.QueryString.ContainsKey("contactId"))
            {
                var contact = GetContact(Int64.Parse(this.NavigationContext.QueryString["contactId"]));
                this.DataContext = contact;
            }

        }

        protected override void OnNavigatedFrom(System.Windows.Navigation.NavigationEventArgs e)
        {
            base.OnNavigatedFrom(e);

            // Save ContactID of the current record we are viewing/editing
            State["DataContextObjectID"] = ((Contact)this.DataContext).ContactID;
        }


        private Contact GetContact(long ContactId)
        {
            // Pull contact out of database
            var contact = (from c in db.Contacts
                           where c.ContactID == ContactId
                           select c).First();
            return contact;
        }

        private void DeleteContact()
        {
            // Delete the Activities for the Leads for the contact
            var activities = from a in db.Activities
                             where a.Lead.Contact == (Contact)DataContext
                             select a;
            db.Activities.DeleteAllOnSubmit(activities);

            // .. and the Leads
            var leads = from l in db.Leads
                        where l.Contact == (Contact)DataContext
                        select l;
            db.Leads.DeleteAllOnSubmit(leads);

            // Delete the Contact from database 
            db.Contacts.DeleteOnSubmit(((Contact)DataContext));
            db.SubmitChanges();
        }

        private CRMDataContext db
        {
            get
            {
                return ((App)App.Current).CRMDataContext;
            }
        }


        private void ApplicationBarEditButton_Click(object sender, EventArgs e)
        {
            // Send OID of selected listbox item
            string dest = "/DetailsPage.xaml?contactId=" + ((Contact)this.DataContext).ContactID;
            NavigationService.Navigate(new Uri(dest, UriKind.Relative));
        }

        private void ApplicationBarDeleteButton_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Really delete this Contact?", "Confirm Deletion", MessageBoxButton.OKCancel) == MessageBoxResult.OK)
            {
                // Delete the Contact from database 
                db.Contacts.DeleteOnSubmit(((Contact)DataContext));
                db.SubmitChanges();

                NavigationService.GoBack();
            }
        }

        private void ApplicationBarNewButton_Click(object sender, EventArgs e)
        {
            string dest = "/DetailsPage.xaml?type=New";
            NavigationService.Navigate(new Uri(dest, UriKind.Relative));
        }

        private void ListBox_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            // Capture selected item data
            var _selectedItem = (sender as ListBox).SelectedItem;

            if (_selectedItem != null)
            {
                // Send OID of selected listbox item
                string dest = "/LeadPivotPage.xaml?oid=" + ((Lead)_selectedItem).LeadID;
                NavigationService.Navigate(new Uri(dest, UriKind.Relative));
            }
        }
    }
}