﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using System.IO.IsolatedStorage;
using System.Threading;

namespace CRMapp
{
    public partial class GeneratorPopup : PhoneApplicationPage
    {
        private DataGenerator _generator;

        public GeneratorPopup()
        {
            InitializeComponent();
            sliderAllCount.Maximum = sliderAllCount.Maximum;
            attTextBlock.Visibility = Visibility.Visible;
        }

        private void CancelButton_Click(object sender, RoutedEventArgs e)
        {
            this.NavigationService.GoBack();
        }

        private void OKButton_Click(object sender, RoutedEventArgs e)
        {
            gRes.Visibility = Visibility.Visible;
            gReq.Visibility = Visibility.Collapsed;

            OKButton.IsEnabled = false;
            OKButton.Visibility = Visibility.Collapsed;
            CancelButton.IsEnabled = true;
            CancelButton.Content = "Stop";

            _generator = new DataGenerator();

            if (((App)Application.Current).CRMDataContext.DatabaseExists())
            {
                _generator.Clear();
            }

            _generator.GeneratedContact +=
                (sender1, e1) => Dispatcher.BeginInvoke(ReadLabels);
            _generator.GenerationComplete +=
                (sender1, e1) =>
                {
                    _generator = null;
                    Dispatcher.BeginInvoke(() => this.NavigationService.GoBack());
                };
            _generator.Committing +=
                (sender1, e1) =>
                    Dispatcher.BeginInvoke(() =>
                    {
                        ReadLabels();
                        cStatus.Text = "Committing...";
                        CancelButton.IsEnabled = false;
                    });

            var c = (int)sliderAllCount.Value;
            using (var store = IsolatedStorageFile.GetUserStoreForApplication())
            {
                var freespace = store.AvailableFreeSpace;
                var needSpace = c * DataGenerator.OneObjectAvgSize;
                if (freespace < needSpace)
                {
                    if (!store.IncreaseQuotaTo(store.Quota + (needSpace - freespace)))
                    {
                        cStatus.Text = "Data generation aborted";
                        CancelButton.Content = "Ok";
                        _generator = null;
                        return;
                    }
                }
            }
            ThreadPool.QueueUserWorkItem(state => _generator.Generate(c));
        }

        private void ReadLabels()
        {
            if (_generator != null)
            {
                cContacts.Text = string.Format("Contacts: {0}", _generator.CountContacts);
                cLeads.Text = string.Format("Leads: {0}", _generator.CountLeads);
                cActivities.Text = string.Format("Activities: {0}", _generator.CountActivities);
                cStatus.Text = "";
            }
        }

        private void Slider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            var v = e.NewValue;
            if (v % 100 > 0)
                ((Slider)sender).Value = ((int)(v / 100)) * 100;
        }

        protected override void OnNavigatedFrom(System.Windows.Navigation.NavigationEventArgs e)
        {
            if (_generator != null)
            {
                _generator.IsComplete = true;
            }
            
            base.OnNavigatedFrom(e);
        }

    }
}