﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;

namespace CRMapp
{
    public static class Utilities
    {
        public static void FillEnums(ItemsControl editElement, Type enumType)
        {
            if (editElement == null) throw new ArgumentNullException("editElement");
            if (enumType == null) throw new ArgumentNullException("enumType");
            if (!typeof (Enum).IsAssignableFrom(enumType))
                throw new InvalidOperationException("enumType must be of Enum type");
            foreach (var value in GetEnumValues(enumType))
                editElement.Items.Add(value);
        }

        public static object[] GetEnumValues(Type enumType)
        {
            if (!enumType.IsEnum)
                throw new ArgumentException("Type '" + enumType.Name + "' is not an enum");

            var values = new List<object>();

            var fields = from field in enumType.GetFields() where field.IsLiteral select field;

            foreach (var field in fields)
            {
                var value = field.GetValue(enumType);
                values.Add(value);
            }

            return values.ToArray();
        }


        public static ObservableCollection<T> ToObservableCollection<T>(this IEnumerable<T> source)
        {
            var collection = new ObservableCollection<T>();
            foreach (var contact in source)
                collection.Add(contact);
            return collection;
        }
    }

    public class VisibilityBoolConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            var val = (bool)value;
            return val ? Visibility.Visible : Visibility.Collapsed;
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }
}