﻿using System;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Globalization;
using System.Data.Linq.Mapping;
using System.Data.Linq;

namespace CRMapp
{
    public enum ActivityPriority : int
    {
        Low,
        Normal,
        High
    }

    public enum ActivityStatus : int
    {
        Complete,
        Incomplete
    }

    public enum ActivityType : int
    {
        Meeting,
        Call
    }

    [Table]
    public class Activity : INotifyPropertyChanged, INotifyPropertyChanging
    {
        private DateTime due;
        private EntityRef<Lead> lead;
        private ActivityType activityType;
        private ActivityPriority priority;
        private ActivityStatus status;
        private string subject;


        // Primary key is automatically indexed
        [Column(IsPrimaryKey = true, IsDbGenerated = true, AutoSync = AutoSync.OnInsert)]
        public long ActivityID { get; set; }

        [Column]
        public DateTime Due
        {
            get { return due; }
            set
            {
                InvokePropertyChanging(new PropertyChangingEventArgs("Due"));
                due = value;
                InvokePropertyChanged(new PropertyChangedEventArgs("Due"));
            }
        }

        [Column]
        public long LeadID { get; set; }

        [Association(Storage = "lead", ThisKey = "LeadID", IsForeignKey=true)]
        public Lead Lead
        {
            get
            {
                return lead.Entity;
            }
            set
            {
                InvokePropertyChanging(new PropertyChangingEventArgs("Lead"));
                lead.Entity = value;
                InvokePropertyChanged(new PropertyChangedEventArgs("Lead"));
                if (value != null)
                    this.LeadID = value.LeadID;
            }
        }

        [Column]
        public ActivityPriority Priority
        {
            get { return priority; }
            set
            {
                InvokePropertyChanging(new PropertyChangingEventArgs("Priority"));
                priority = value;
                InvokePropertyChanged(new PropertyChangedEventArgs("Priority"));
            }
        }

        [Column]
        public ActivityStatus Status
        {
            get { return status; }
            set
            {
                InvokePropertyChanging(new PropertyChangingEventArgs("Status"));
                status = value;
                InvokePropertyChanged(new PropertyChangedEventArgs("Status"));
            }
        }

        [Column]
        public ActivityType ActivityType
        {
            get { return activityType; }
            set
            {
                InvokePropertyChanging(new PropertyChangingEventArgs("ActivityType"));
                activityType = value;
                InvokePropertyChanged(new PropertyChangedEventArgs("ActivityType"));
            }
        }

        [Column]
        public string Subject
        {
            get { return subject ?? ""; }
            set
            {
                InvokePropertyChanging(new PropertyChangingEventArgs("Subject"));
                subject = value;
                InvokePropertyChanged(new PropertyChangedEventArgs("Subject"));
                System.Diagnostics.Debug.WriteLine(this.ToString() + " loaded");
            }
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        #region INotifyPropertyChanging Members

        public event PropertyChangingEventHandler PropertyChanging;

        #endregion

        public void Deallocate()
        {
            var leads = db.Leads.Where(l => l.NextStep == this);
            foreach (var l in leads)
            {
                l.NextStep = null;
            }

            Activity activity = (from a in db.Activities
                                 where a.ActivityID == this.ActivityID
                                 select a).FirstOrDefault();

            db.Activities.DeleteOnSubmit(activity);

            db.SubmitChanges();
        }

        public override string ToString()
        {
            return string.Format(CultureInfo.CurrentCulture, "{0} {1}. {2}. {3}", Due, Subject,
                                 Lead != null ? Lead.Name : "",
                                 Lead != null && Lead.Contact != null ? Lead.Contact.LastName : "");
        }

        private CRMDataContext db
        {
            get
            {
                return ((App)App.Current).CRMDataContext;
            }
        }

        private void InvokePropertyChanged(PropertyChangedEventArgs e)
        {
            var handler = PropertyChanged;
            if (handler != null) handler(this, e);
        }

        private void InvokePropertyChanging(PropertyChangingEventArgs e)
        {
            var handler = PropertyChanging;
            if (handler != null) handler(this, e);
        }
    }
}
