﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Collections.Generic;
using System.Linq;
using System.Data.Linq.Mapping;
using System.Collections.ObjectModel;
using System.Data.Linq;
using Microsoft.Phone.Data.Linq.Mapping;

namespace CRMapp
{
    [Table]
    [Index (Name="ContactID_Idx", Columns="ContactID", IsUnique=false)]
    public class Lead : INotifyPropertyChanged, INotifyPropertyChanging
    {
        private int amount;
        private EntityRef<Contact> contact;
        private DateTime expectedClose;
        private string name;
        private double probability;
        private EntitySet<Activity> activities = new EntitySet<Activity>();
        private EntityRef<Activity> nextStep;


        private CRMDataContext db
        {
            get
            {
                return ((App)App.Current).CRMDataContext;
            }
        }

        // Primary key is automatically indexed
        [Column(IsPrimaryKey = true, IsDbGenerated = true, AutoSync = AutoSync.OnInsert)]
        public long LeadID { get; set; }


        [Column]
        public long ContactID {get; set; }

        [Association(Storage = "contact", ThisKey = "ContactID", IsForeignKey=true)]
        public Contact Contact
        {
            get { return this.contact.Entity; }
            set 
            {
                InvokePropertyChanging(new PropertyChangingEventArgs("Contact"));
                this.contact.Entity = value;
                InvokePropertyChanged(new PropertyChangedEventArgs("Contact"));
                if (value != null)
                    this.ContactID = value.ContactID;
            }
        }

        [Association(Storage = "activities", OtherKey = "LeadID")]
        public EntitySet<Activity> Activities
        {
            get
            {
                return this.activities;
            }
            set
            {
                InvokePropertyChanging(new PropertyChangingEventArgs("Activities"));
                this.activities.Assign(value);
                InvokePropertyChanged(new PropertyChangedEventArgs("Activities"));
            }
        }

        [Column]
        public int Amount
        {
            get { return amount; }
            set
            {
                InvokePropertyChanging(new PropertyChangingEventArgs("Amount"));
                amount = value;
                InvokePropertyChanged(new PropertyChangedEventArgs("Amount"));
            }
        }

        [Column]
        public DateTime ExpectedClose
        {
            get { return expectedClose; }
            set
            {
                InvokePropertyChanging(new PropertyChangingEventArgs("ExpectedClose"));
                expectedClose = value;
                InvokePropertyChanged(new PropertyChangedEventArgs("ExpectedClose"));
            }
        }

        [Column]
        public string Name
        {
            get { return name ?? ""; }
            set
            {
                InvokePropertyChanging(new PropertyChangingEventArgs("Name"));
                name = value;
                InvokePropertyChanged(new PropertyChangedEventArgs("Name"));
                System.Diagnostics.Debug.WriteLine(this.ToString() + " loaded");
            }
        }

        [Column]
        public long NextStepID { get; set; }

        [Association(Storage = "nextStep", ThisKey = "NextStepID")]
        public Activity NextStep
        {
            get { return nextStep.Entity; }
            set
            {
                InvokePropertyChanging(new PropertyChangingEventArgs("NextStep"));
                nextStep.Entity = value;
                InvokePropertyChanged(new PropertyChangedEventArgs("NextStep"));
                if (value != null)
                {
                    this.NextStepID = value.ActivityID;
                }
            }
        }

        [Column]
        public double Probability
        {
            get { return probability; }
            set
            {
                InvokePropertyChanging(new PropertyChangingEventArgs("Probability"));
                probability = value;
                InvokePropertyChanged(new PropertyChangedEventArgs("Probability"));
            }
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        #region INotifyPropertyChanging Members

        public event PropertyChangingEventHandler PropertyChanging;

        #endregion

        public void Deallocate()
        {
            var activities = from a in db.Activities
                             where a.Lead == this
                             select a;
            db.Activities.DeleteAllOnSubmit(activities);

            Lead lead = (from w in db.Leads
                         where w.LeadID == this.LeadID
                         select w).FirstOrDefault();

            db.Leads.DeleteOnSubmit(lead);

            db.SubmitChanges();
        }

        public override string ToString()
        {
            var contact = (from c in db.Contacts
                           where c.ContactID == this.ContactID
                           select c).FirstOrDefault();
            return string.Format("{0}: {1}", contact != null ? contact.LastName : "", Name);
        }

        private void InvokePropertyChanged(PropertyChangedEventArgs e)
        {
            var handler = PropertyChanged;
            if (handler != null) handler(this, e);
        }

        private void InvokePropertyChanging(PropertyChangingEventArgs e)
        {
            var handler = PropertyChanging;
            if (handler != null) handler(this, e);
        }
    }
}
