﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;

namespace CRMapp
{
    public partial class DetailsPage : PhoneApplicationPage
    {
        private bool setTBFocus = false;

        private bool isNewRecord = false;

        public DetailsPage()
        {
            InitializeComponent();

            SupportedOrientations = SupportedPageOrientation.Portrait | SupportedPageOrientation.Landscape;

            this.LayoutUpdated += (s, e) =>
                {
                    // Set the focus on first layout, if needed
                    if (setTBFocus)
                    {
                        setTBFocus = false;
                        this.tbFirstName.Focus();
                    }
                };
        }

        protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);

            // For tombstoning, save the contactId of a particular record in the State object on OnNavigatedFrom
            // Restore it in OnNavigatedTo and extract the correct one from the database
            if (State.ContainsKey("DataContextObjectID"))
            {
                int cid = (int)State["DataContextObjectID"];
                this.DataContext = GetContact(cid);
                State.Remove("DataContextObjectID");
            }

            if (State.ContainsKey("isNewRecord"))
            {
                isNewRecord = (bool)State["isNewRecord"];
                State.Remove("isNewRecord");
            }

            // Have we come here by navigating from the MainPage after selecting a record in the main list?
            if (this.NavigationContext.QueryString.ContainsKey("contactId"))
            {
                var contact = GetContact(Int64.Parse(this.NavigationContext.QueryString["contactId"]));
                this.DataContext = contact;
            }

            // Have we come here by navigating from the MainPage for a 'New Contact'?
            if (this.NavigationContext.QueryString.ContainsKey("type"))
            {
                if (this.NavigationContext.QueryString["type"] == "New")
                {
                    isNewRecord = true;
                }
            }

            if (isNewRecord)
            {
                Contact newC = new Contact();

                // Adding new record to Database
                db.Contacts.InsertOnSubmit(newC);
                db.SubmitChanges(); 

                this.DataContext = newC;

                ((ApplicationBarIconButton)ApplicationBar.Buttons[1]).IconUri = new Uri("Images/appbar.cancel.rest.png", UriKind.Relative);
                ((ApplicationBarIconButton)ApplicationBar.Buttons[1]).Text = "cancel";

                setTBFocus = true;

                if (State.ContainsKey("FirstName"))
                {
                    tbFirstName.Text = (string)State["FirstName"];
                    State.Remove("FirstName");
                }
                if (State.ContainsKey("Lastname"))
                {
                    tbFirstName.Text = (string)State["Lastname"];
                    State.Remove("Lastname");
                }
                if (State.ContainsKey("Company"))
                {
                    tbFirstName.Text = (string)State["Company"];
                    State.Remove("Company");
                }
                if (State.ContainsKey("Email"))
                {
                    tbFirstName.Text = (string)State["Email"];
                    State.Remove("Email");
                }
                if (State.ContainsKey("Address"))
                {
                    tbFirstName.Text = (string)State["Address"];
                    State.Remove("Address");
                }
            }
            else
            {
                // View Details mode - just one apbar button needed
                ApplicationBar.Buttons.RemoveAt(0);
                ((ApplicationBarIconButton)ApplicationBar.Buttons[0]).IconUri = new Uri("Images/appbar.delete.rest.png", UriKind.Relative);
                ((ApplicationBarIconButton)ApplicationBar.Buttons[0]).Text = "delete";
            }
        }

        private Contact GetContact(long ContactId)
        {
            // Pull contact out of database
            var contact = (from c in db.Contacts
                           where c.ContactID == ContactId
                           select c).First();
            return contact;
        }

        protected override void OnNavigatedFrom(System.Windows.Navigation.NavigationEventArgs e)
        {
            base.OnNavigatedFrom(e);

            State["isNewRecord"] = isNewRecord;

            if (isNewRecord)
            {
                // Unsaved new record
                DeleteContact();

                // Save state of work in progress to restore on reactivation
                State["FirstName"] = tbFirstName.Text;
                State["Lastname"] = tbLastName.Text;
                State["Company"] = tbCompany.Text;
                State["Email"] = tbEmail.Text;
                State["Address"] = tbAddress.Text;
            }
            else
            {
                // Save ContactID of the current record we are viewing/editing
                State["DataContextObjectID"] = ((Contact)this.DataContext).ContactID;
            }
        }

        private void DeleteContact()
        {
            // Delete the Contact from database 
            db.Contacts.DeleteOnSubmit(((Contact)DataContext));
            db.SubmitChanges();
        }

        private CRMDataContext db
        {
            get
            {
                return ((App)App.Current).CRMDataContext;
            }
        }

        // Handle navigating back to content in the two frames
        private void PhoneApplicationPage_BackKeyPress(object sender, System.ComponentModel.CancelEventArgs e)
        {
            if (!ValidateAndSaveContact())
                e.Cancel = true;
        }

        // This button is only active when adding a new contact
        private void CancelButton_Click(object sender, EventArgs e)
        {
            // Delete the Contact
            DeleteContact();

            NavigationService.GoBack();
        }

        private void OkButton_Click(object sender, EventArgs e)
        {
            if (ValidateAndSaveContact())
                NavigationService.GoBack();
        }

        private bool ValidateAndSaveContact()
        {
            bool isValid = true;
            // Ensure focused TextBox updates the data source
            App.UpdateTextbox();

            if (((Contact)DataContext).IsValid)
            {
                // Save database changes
                db.SubmitChanges();

                // ensure new record flag is clear (for tombstoning)
                isNewRecord = false;
            }
            else
            {
                string msg = "";
                foreach (var item in ((Contact)DataContext).GetErrors)
                {
                    msg += (item + " ");
                }
                MessageBox.Show(msg, "Errors", MessageBoxButton.OK);
                isValid = false;
            }

            return isValid;
        }
    }
}