﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using Microsoft.Phone.Controls;
using System.Linq;
using System.Windows.Navigation;
using System.Collections;
using System.Collections.ObjectModel;
using System.Data.Linq;
using Microsoft.Phone.Data.Linq;

namespace CRMapp
{
    public partial class MainPage : PhoneApplicationPage
    {
        object _selectedItem;
        private string detailsQueryString = string.Empty;
        private ObservableCollection<Contact> searchcontacts = null;

        public MainPage()
        {
            InitializeComponent();

            SupportedOrientations = SupportedPageOrientation.PortraitOrLandscape;

            ContactsListBox.Loaded += new RoutedEventHandler(ContactsListBox_Loaded);

            // Get contacts data from the database
            RefreshContacts();
        }

        private CRMDataContext db
        {
            get
            {
                return ((App)App.Current).CRMDataContext;
            }
        }

        private bool IsSearchableState
        {
            get { return (tbSearch.Text.Length == 0); }
        }

        private void ListBoxOne_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            // Capture selected item data
            _selectedItem = (sender as ListBox).SelectedItem;

            if (_selectedItem != null)
            {
                // Send ID of selected contact
                string dest = "/ContactPivotPage.xaml?contactId=" + ((Contact)_selectedItem).ContactID;
                NavigationService.Navigate(new Uri(dest, UriKind.Relative));
            }
        }

        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            base.OnNavigatedFrom(e);

            // Remember scroll offset on ListBox, and state of search box
            try   
            {
                ScrollViewer viewer = ((System.Windows.Media.VisualTreeHelper.GetChild(ContactsListBox, 0) as FrameworkElement).FindName("ScrollViewer") as ScrollViewer);
                State["scrollOffset"] = viewer.VerticalOffset;

                State["tbSearchVisible"] = (SearchPanel.Visibility == System.Windows.Visibility.Visible);
                State["tbSearch"] = tbSearch.Text;

                // Are we in a search?
                State["Issearchcontacts"] = (searchcontacts != null);
            } 
            catch { }
        }

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);

            // Reset the contacts list after returning from Data generation, Clear or Edit
            RefreshContacts();
            
            detailsQueryString = string.Empty;

            // Although if we are in search mode (we've searched, selected a contact in the list, now come back
            if (searchcontacts != null)
            {
                // ... set the ListBox to the filtered list
                ContactsListBox.ItemsSource = searchcontacts;
            }

            // Are we in a search?
            object sc;
            if (State.TryGetValue("Issearchcontacts", out sc))
            {
                State.Remove("Issearchcontacts");
                if ((bool)sc)
                {
                    tbSearch.Text = string.Empty;
                    object tbText;
                    if (State.TryGetValue("tbSearch", out tbText))
                    {
                        State.Remove("tbSearch");
                        tbSearch.Text = (string)tbText;
                    }
                }
            }

            object isSearchVisible;
            if (State.TryGetValue("tbSearchVisible", out isSearchVisible))
            {
                State.Remove("tbSearchVisible");

                if ((bool)isSearchVisible == true)
                {
                    SearchPanel.Visibility = System.Windows.Visibility.Visible;
                }
            }

            object offset; 
            // Return scroll offset
            if (State.TryGetValue("scrollOffset", out offset))
            {
                scrollOffset = (double)offset;
                State.Remove("scrollOffset");
            }
        }

        private double scrollOffset;

        void ContactsListBox_Loaded(object sender, RoutedEventArgs e)
        {
            try
            {
                ScrollViewer viewer = ((System.Windows.Media.VisualTreeHelper.GetChild(ContactsListBox, 0) as FrameworkElement).FindName("ScrollViewer") as ScrollViewer);
                viewer.ScrollToVerticalOffset(scrollOffset);
            }
            catch { }
        }

        private void NewContactButton_Click(object sender, EventArgs e)
        {
            string dest = "/DetailsPage.xaml?type=New";
            NavigationService.Navigate(new Uri(dest, UriKind.Relative));
        }

        private void SearchButton_Click(object sender, EventArgs e)
        {
            if (SearchPanel.Visibility == System.Windows.Visibility.Collapsed)
            {
                SearchPanel.Visibility = System.Windows.Visibility.Visible;
                tbSearch.Focus();
                this.tbSearch.ItemsSource = this.ContactsListBox.ItemsSource;
            }
            else
            {
                tbSearch.Text = String.Empty;
                SearchPanel.Visibility = System.Windows.Visibility.Collapsed;
                RefreshContacts();
            }
        }

        private void GenerateMenuItem_Click(object sender, EventArgs e)
        {
            // navigate
            this.NavigationService.Navigate(new Uri("/GeneratorPopup.xaml", UriKind.Relative));
        }

        private void ClearMenuItem_Click(object sender, EventArgs e)
        {
            // navigate
            this.NavigationService.Navigate(new Uri("/ClearPopup.xaml", UriKind.Relative));
        }

        private void RefreshContacts()
        {
           if (!IsSearchableState) return;

           var contactlist = from c in db.Contacts
                             orderby c.LastName, c.FirstName
                             select c;
           this.ContactsListBox.ItemsSource = contactlist;
        }

        private void btnSearchCancel_Click(object sender, RoutedEventArgs e)
        {
            tbSearch.Text = String.Empty;
            SearchPanel.Visibility = System.Windows.Visibility.Collapsed;
            RefreshContacts();
            searchcontacts = null;
        }

        private void tbSearch_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count > 0)
            {
                ContactsListBox.ScrollIntoView(e.AddedItems[0]);
                Dispatcher.BeginInvoke(()=>
                    ContactsListBox.Focus()); // Hide keyboard by removing focus from AutoCompleteBox
            }
        }
    }
}