using System.ComponentModel;
using System.Linq;
using System.Globalization;
using System.Collections.Generic;
using System;
using System.Data.Linq.Mapping;
using System.Data.Linq;

namespace CRMapp
{
    // Defines a database table 
    [Table]
    public class Contact : INotifyPropertyChanged, INotifyPropertyChanging
    {
        // Fields 
        private string address;
        private string company;
        private string email;
        private string firstName;
        private string lastName;
        private EntitySet<Lead> leads = new EntitySet<Lead>();

        // Primary key is automatically indexed
        [Column(IsPrimaryKey=true, IsDbGenerated=true, AutoSync=AutoSync.OnInsert)]
        public long ContactID { get; set; }

        [Column]
        public string Address
        {
            get { return address; }
            set
            {
                InvokePropertyChanging(new PropertyChangingEventArgs("Address"));
                address = value;
                InvokePropertyChanged(new PropertyChangedEventArgs("Address"));
            }
        }

        [Column]
        public string Company
        {
            get { return company; }
            set
            {
                InvokePropertyChanging(new PropertyChangingEventArgs("Company"));
                company = value;
                InvokePropertyChanged(new PropertyChangedEventArgs("Company"));
            }
        }

        [Column]
        public string Email
        {
            get { return email; }
            set
            {
                InvokePropertyChanging(new PropertyChangingEventArgs("Email"));
                email = value;
                InvokePropertyChanged(new PropertyChangedEventArgs("Email"));
            }
        }

        [Column]
        public string FirstName
        {
            get { return firstName; }
            set
            {
                InvokePropertyChanging(new PropertyChangingEventArgs("FirstName"));
                firstName = value;
                InvokePropertyChanged(new PropertyChangedEventArgs("FirstName"));
            }
        }

        [Column]
        public string LastName
        {
            get { return lastName; }
            set
            {
                if ((this.lastName != value))
                {
                    InvokePropertyChanging(new PropertyChangingEventArgs("LastName"));
                    this.lastName = value;
                    InvokePropertyChanged(new PropertyChangedEventArgs("LastName"));
                }

            }
        }

        [Association(Storage = "leads", OtherKey = "ContactID")]
        public EntitySet<Lead> Leads
        {
            get
            {
                return this.leads; 
            }
            set
            {
                InvokePropertyChanging(new PropertyChangingEventArgs("Leads"));
                this.leads.Assign(value);
                InvokePropertyChanged(new PropertyChangedEventArgs("Leads"));
            }
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        #region INotifyPropertyChanging Members

        public event PropertyChangingEventHandler PropertyChanging;

        #endregion

        public bool IsValid
        {
            get
            {
                if (this.lastName == null || this.lastName.Length == 0) return false;
                return true;
            }
        }

        public List<string> GetErrors
        {
            get
            {
                List<string> errors = new List<string>();
                if (this.lastName == null || this.lastName.Length == 0) errors.Add("Last Name is required");

                return errors;
            }
        }

        public override string ToString()
        {
            return string.Format(CultureInfo.CurrentCulture, "{0} {1}", FirstName, LastName);
        }

        private void InvokePropertyChanged(PropertyChangedEventArgs e)
        {
            var handler = PropertyChanged;
            if (handler != null) handler(this, e);
        }

        private void InvokePropertyChanging(PropertyChangingEventArgs e)
        {
            var handler = PropertyChanging;
            if (handler != null) handler(this, e);
        }

        private CRMDataContext db
        {
            get
            {
                return ((App)App.Current).CRMDataContext;
            }
        }


    }

}