﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.IO;
using System.Collections.ObjectModel;
using System.Threading;
using Microsoft.Phone.Shell;
using Microsoft.Phone.Notification;
using WindowsPhone.Recipes.Push.Client.PushServiceReference;
using WindowsPhone.Recipes.Push.Client.Controls;

namespace WindowsPhone.Recipes.Push.Client.Views
{
    public partial class InboxView : UserControl
    {
        #region Fields

        private readonly ObservableCollection<string> _rawMessages = new ObservableCollection<string>();
        private ShellTileSchedule _tileSchedule;

        #endregion

        #region Properties

        public ObservableCollection<string> RawMessages
        {
            get { return _rawMessages; }
        }

        #endregion

        public InboxView()
        {
            DataContext = this;
            InitializeComponent();

            Loaded += InboxView_Loaded;
            Unloaded += InboxView_Unloaded;
        }

        private void InboxView_Unloaded(object sender, RoutedEventArgs e)
        {
            UnregisterRawNotification();
        }

        private void InboxView_Loaded(object sender, RoutedEventArgs e)
        {
            RegisterRawNotification();
        }

        private void RegisterRawNotification()
        {
            var context = PushContext.Current;
            if (context != null)
            {
                context.RawNotification += context_RawNotification;
            }
        }

        private void UnregisterRawNotification()
        {
            var context = PushContext.Current;
            if (context != null)
            {
                context.RawNotification -= context_RawNotification;
            }
        }

        private void context_RawNotification(object sender, HttpNotificationEventArgs e)
        {
            try
            {
                using (var stream = new StreamReader(e.Notification.Body))
                {
                    var rawMessage = stream.ReadToEnd();
                    _rawMessages.Insert(0, rawMessage);
                    if ("AskToPin".CompareTo(rawMessage) == 0)
                    {
                        AskToPin();
                    }

                }
            }
            catch (Exception ex)
            {
                ex.Show();
            }
        }

        private void AskToPin()
        {
            NotificationBox.Show("Important", "Please pin your application to Start Screen so this application can work properly.");
        }

    }
}
