﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Web;
using System.Text;
using System.Collections.ObjectModel;
using WindowsPhone.Recipes.Push.Messasges;

namespace MyPushNotificationWCFServiceWebRole
{
    // NOTE: You can use the "Rename" command on the "Refactor" menu to change the class name "Service1" in code, svc and config file together.
    public class PushService : IPushService
    {
        // This is NOT the way to do it in a real Azure app - use Azure table Storage since that is accessible
        // across multiple instances of this WebRole
        static Collection<Subscription> SubscriptionCollection = new Collection<Subscription>();

        /// <summary>
        /// This method is called by mobile apps
        /// </summary>
        /// <param name="userName"></param>
        /// <param name="channelUri"></param>
        public void Register(string userName, Uri channelUri)
        {
            var existingSubscription = (from s in SubscriptionCollection
                                        where s.UserName == userName
                                        select s).FirstOrDefault();
            if (existingSubscription != null)
            {
                // Already know this user
                existingSubscription.ChannelUri = channelUri;
            }
            else
            {
                // New subscription
                SubscriptionCollection.Add(new Subscription(userName, channelUri));
            }
        }


        /// <summary>
        /// This method is called by the desktop app that we use in this demo to drive new notifications
        /// </summary>
        /// <param name="msgdata"></param>
        public void SendRaw(string msgdata)
        {
            foreach (var subs in SubscriptionCollection)
            {
                var raw = new RawPushNotificationMessage(MessageSendPriority.High)
                {
                    RawData = Encoding.ASCII.GetBytes(msgdata)
                };

                raw.SendAsync(subs.ChannelUri);
            }
        }

        /// <summary>
        /// This method is called by the desktop app that we use in this demo to drive new notifications
        /// </summary>
        /// <param name="title"></param>
        /// <param name="msgdata"></param>
        public void SendToast(string title, string msgdata)
        {
            foreach (var subs in SubscriptionCollection)
            {
                var toast = new ToastPushNotificationMessage(MessageSendPriority.High)
                {
                    Title = title,
                    SubTitle = msgdata
                };

                toast.SendAsync(subs.ChannelUri);
            }
        }

        /// <summary>
        /// This method is called by the desktop app that we use in this demo to drive new notifications
        /// Note that in this demo, we are not setting Tile backgrounds or any of the new mango features
        /// but this sampe could easily be extended to do so.
        /// </summary>
        /// <param name="title"></param>
        /// <param name="count"></param>
        /// <param name="imageURI"></param>
        public void SendTileUpdate(string title, int count, Uri imageURI)
        {
            foreach (var subs in SubscriptionCollection)
            {
                var tile = new TilePushNotificationMessage(MessageSendPriority.High)
                {
                    Title = title,
                    BackgroundImageUri = imageURI,
                    Count = count
                };

                tile.SendAsync(subs.ChannelUri);
            }
        }
    }
}
