﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Device.Location;
using Microsoft.Phone.Controls.Maps;

namespace MapsEmulatorTest
{
    public class MapDataViewModel : INotifyPropertyChanged
    {
        #region Consts
        private const double DefaultZoomLevel = 17.0;
        private const double MaxZoomLevel = 21.0;
        private const double MinZoomLevel = 1.0;
        #endregion

        private readonly CredentialsProvider _credentialsProvider = new ApplicationIdCredentialsProvider(App.Id);

        private static readonly GeoCoordinate DefaultLocation = new GeoCoordinate(53.777719, -1.572016);

        private GeoCoordinate _center;

        private double _zoom;



        public GeoCoordinate Center
        {
            get { return _center; }
            set
            {
                if (_center != value)
                {
                    _center = value;
                    NotifyPropertyChanged("Center");
                }
            }
        }

        public CredentialsProvider CredentialsProvider
        {
            get
            {
                return _credentialsProvider;
            }
        }

        /// <summary>
        /// Gets or sets the map zoom level.
        /// </summary>
        public double Zoom
        {
            get { return _zoom; }
            set
            {
                var coercedZoom = Math.Max(MinZoomLevel, Math.Min(MaxZoomLevel, value));
                if (_zoom != coercedZoom)
                {
                    _zoom = value;
                    NotifyPropertyChanged("Zoom");
                }
            }

        }

        public void CenterLocation()
        {
            // TODO : Center current location.
            Center = DefaultLocation;

            Zoom = DefaultZoomLevel;
        }

        #region Property Changed

        private void NotifyPropertyChanged(string propertyName)
        {
            PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        public event PropertyChangedEventHandler PropertyChanged = delegate { };

        #endregion
    }
}
