﻿#define DEBUG_AGENT

using Microsoft.Phone.Scheduler;

using Microsoft.Phone.Shell;
using System;
using System.IO;
using System.IO.IsolatedStorage;
using System.Collections.Generic;
using System.Device.Location;

namespace LocationTaskAgent
{
    public class ScheduledAgent : ScheduledTaskAgent
    {

        #region Isolated Storage - using the settings store


        private void saveTextToIsolatedStorage(string filename, string text)
        {
            using (IsolatedStorageFile isf =
                          IsolatedStorageFile.GetUserStoreForApplication())
            {
                using (IsolatedStorageFileStream rawStream =
                                 isf.CreateFile(filename))
                {
                    StreamWriter writer = new StreamWriter(rawStream);
                    writer.Write(text);
                    writer.Close();
                }
            }
        }

        private bool loadTextFromIsolatedStorage(string filename, out string result)
        {
            result = "";
            using (IsolatedStorageFile isf =
                IsolatedStorageFile.GetUserStoreForApplication())
            {
                if (isf.FileExists(filename))
                {
                    try
                    {
                        using (IsolatedStorageFileStream rawStream =
                            isf.OpenFile(filename, System.IO.FileMode.Open))
                        {
                            StreamReader reader = new StreamReader(rawStream);
                            result = reader.ReadToEnd();
                            reader.Close();
                        }
                    }
                    catch
                    {
                        return false;
                    }
                }
                else
                {
                    return false;
                }
            }
            return true;
        }


        private void saveTextToIsolatedStorageSettings(string filename, string text)
        {
            IsolatedStorageSettings isolatedStore = IsolatedStorageSettings.ApplicationSettings;
            isolatedStore.Remove(filename);
            isolatedStore.Add(filename, text);
            isolatedStore.Save();
        }

        private bool loadTextFromIsolatedStorageSettings(string filename, out string result)
        {
            IsolatedStorageSettings isolatedStore = IsolatedStorageSettings.ApplicationSettings;

            result = "";
            try
            {
                result = (string)isolatedStore[filename];
            }
            catch
            {
                return false;
            }
            return true;
        }

        #endregion

        /// <summary>
        /// Agent that runs a scheduled task
        /// </summary>
        /// <param name="task">
        /// The invoked task
        /// </param>
        /// <remarks>
        /// This method is called when a periodic or resource intensive task is invoked
        /// </remarks>
        protected override void OnInvoke(ScheduledTask task)
        {
            string message ="";

            string logString = "";

            if (loadTextFromIsolatedStorage("Log", out logString))
            {
                message = "Loaded";
            }
            else
            {
                message = "Initialised";
            }

            DateTime timeStamp = DateTime.Now;
            string timeStampString = timeStamp.ToShortDateString() + " " + timeStamp.ToShortTimeString() + System.Environment.NewLine;

            GeoCoordinateWatcher watcher = new GeoCoordinateWatcher();
            watcher.Start();
            string positionString = watcher.Position.Location.ToString() + System.Environment.NewLine;

            logString = logString + timeStampString + positionString;

            saveTextToIsolatedStorage("Log", logString);

            ShellToast toast = new ShellToast();
            toast.Title = "Captain's Log";
            toast.Content = message;
            toast.Show();

#if DEBUG_AGENT
            ScheduledActionService.LaunchForTest(task.Name, TimeSpan.FromSeconds(60));
#endif
            NotifyComplete();
        }
    }
}
