using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;

using Microsoft.Devices.Sensors;

namespace CheeseRoller
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class CheeseRollerGame : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        // Game World

        Texture2D cheeseTexture;
        Rectangle cheeseDrawPosition;
        Vector2 cheesePosition;
        Vector2 cheeseSpeed;
        float friction = 0.99f;

        public CheeseRollerGame()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            // Frame rate is 30 fps by default for Windows Phone.
            TargetElapsedTime = TimeSpan.FromTicks(333333);

            // Tell XNA we can only support Portrait orientation
            // Can support a range of orientations by ORing 
            // together orientation values
            graphics.SupportedOrientations = DisplayOrientation.Portrait;

            // Set up hardware scaling of the display area
            graphics.PreferredBackBufferWidth = 480;
            graphics.PreferredBackBufferHeight = 800;

            Accelerometer accel = new Accelerometer();
            accel.ReadingChanged +=
                   new EventHandler<AccelerometerReadingEventArgs>
                                            (accel_ReadingChanged);
            accel.Start();
        }

        object accelLock = new object();

        Vector3 accelReading;

        void accel_ReadingChanged(object sender, AccelerometerReadingEventArgs e)
        {
            lock (accelLock)
            {
                accelReading.X = (float)e.X;
                accelReading.Y = (float)e.Y;
                accelReading.Z = (float)e.Z;
            }
        }


        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // Create the draw rectangle 
            // Cheese should be a 12th the size of the screen
            cheeseDrawPosition = new Rectangle(
                0, 0, // top left hand corner of the screen
                GraphicsDevice.Viewport.Width / 9, // width
                GraphicsDevice.Viewport.Width / 9  // height
                );

            TouchPanel.EnabledGestures = GestureType.Flick;

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            cheeseTexture = Content.Load<Texture2D>("Edam");
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            cheesePosition = cheesePosition + cheeseSpeed ;

            cheeseSpeed = cheeseSpeed * friction;

            lock (accelLock)
            {
                Vector2 cheeseAcceleration = new Vector2(accelReading.X, -accelReading.Y);
                cheeseSpeed = cheeseSpeed + (cheeseAcceleration / 5f);
            }

            cheeseDrawPosition.X = (int)(cheesePosition.X + 0.5f);
            cheeseDrawPosition.Y = (int)(cheesePosition.Y + 0.5f);

            if (cheeseDrawPosition.X < 0)
                cheeseSpeed.X = Math.Abs(cheeseSpeed.X);
            if (cheeseDrawPosition.Right > GraphicsDevice.Viewport.Width)
                cheeseSpeed.X = -Math.Abs(cheeseSpeed.X);

            if (cheeseDrawPosition.Y < 0)
                cheeseSpeed.Y = Math.Abs(cheeseSpeed.Y);
            if (cheeseDrawPosition.Bottom > GraphicsDevice.Viewport.Height)
                cheeseSpeed.Y = -Math.Abs(cheeseSpeed.Y);

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            spriteBatch.Begin();

            spriteBatch.Draw(cheeseTexture, cheeseDrawPosition, Color.White);

            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
