﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;

using Microsoft.Devices;
using Microsoft.Xna.Framework.Media;
using System.Threading;
using System.Windows.Media.Imaging;


namespace FunkyCamera
{
    public partial class MainPage : PhoneApplicationPage
    {

        PhotoCamera camera;
        private static ManualResetEvent captureEvent = new ManualResetEvent(true);
        private static ManualResetEvent pauseFramesEvent = new ManualResetEvent(true);
        private WriteableBitmap wb;
        private Thread ARGBFramesThread;
        private bool pumpARGBFrames;

        // Constructor
        public MainPage()
        {
            InitializeComponent();
        }

        void PumpARGBFrames()
        {

            int[] ARGBPx = new int[640 * 480];

            try
            {

                while (pumpARGBFrames)
                {
                    // Wait for an image to be ready
                    captureEvent.WaitOne();
                    pauseFramesEvent.WaitOne();

                    //Copies the current viewfinder frame into a buffer for further manipulation
                    camera.GetPreviewBufferArgb32(ARGBPx);

                    //Conversion to funky colours
                    for (int i = 0; i < ARGBPx.Length; i++)
                    {
                        ARGBPx[i] = FunkyColor(ARGBPx[i]);
                    }

                    // Release the camera
                    pauseFramesEvent.Reset();

                    // Copy the processed image back onto the screen
                    Deployment.Current.Dispatcher.BeginInvoke(delegate()
                    {
                        //Copy to WriteableBitmap
                        ARGBPx.CopyTo(wb.Pixels, 0);
                        wb.Invalidate();
                        // Make ready for the next image
                        pauseFramesEvent.Set();
                    });
                }

            }

            catch (Exception e)
            {
                this.Dispatcher.BeginInvoke(delegate()
                {
                    //Display error message
                    MessageBox.Show(e.Message);
                });
            }
        }


        internal int ColorToGray(int color)
        {
            int gray = 0;

            int a = color >> 24;
            int r = (color & 0x00ff0000) >> 16;
            int g = (color & 0x0000ff00) >> 8;
            int b = (color & 0x000000ff);

            if ((r == g) && (g == b))
            {
                gray = color;
            }
            else
            {
                // Calculate for the illumination
                // I =(int)(0.109375*R + 0.59375*G + 0.296875*B + 0.5)
                int i = (7 * r + 38 * g + 19 * b + 32) >> 6;

                gray = ((a & 0xFF) << 24) | ((i & 0xFF) << 16) | ((i & 0xFF) << 8) | (i & 0xFF);
            }

            return gray;
        }

        int redOffset = 150;
        int greenOffset = 170;
        int blueOffset = 200;

        internal int FunkyColor(int color)
        {
            int a = color >> 24;
            int r = (color & 0x00ff0000) >> 16;
            int g = (color & 0x0000ff00) >> 8;
            int b = (color & 0x000000ff);

            r += redOffset;
            g += greenOffset;
            b += blueOffset;
            return ((a & 0xFF) << 24) | ((r & 0xFF) << 16) | ((g & 0xFF) << 8) | (b & 0xFF);
        }


        protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
        {
            camera = new Microsoft.Devices.PhotoCamera();

            // Create the destination for the processed image
            wb = new WriteableBitmap(640, 480);
            this.ProcessedImage.Source = wb;

            // Start the image pump when the camera is ready
            camera.Initialized +=
                new EventHandler<CameraOperationCompletedEventArgs>(camera_Initialized);

            //Set the VideoBrush source to the camera
            viewfinderBrush.SetSource(camera);
        }

        void camera_Initialized(object sender, CameraOperationCompletedEventArgs e)
        {
            pumpARGBFrames = true;
            ARGBFramesThread = new System.Threading.Thread(PumpARGBFrames);
            ARGBFramesThread.Start();
        }
    }
}