﻿using System;
using System.Linq;
using System.Windows;
using Microsoft.Phone.Controls;
using System.Device.Location;
using Microsoft.Phone.UserData;
using Microsoft.Phone.Controls.Maps;
using MapsEmulatorTest.GeocodeServiceReference;

namespace MapsEmulatorTest
{
    public partial class MainPage : PhoneApplicationPage
    {

        MapDataViewModel viewModel = new MapDataViewModel();
        GeoCoordinateWatcher gcw = new GeoCoordinateWatcher();
        Contacts cons = new Contacts();

        // Constructor
        public MainPage()
        {
            InitializeComponent();

            this.Loaded += (s, a) =>
                {
                    MyMap.DataContext = viewModel;

                    gcw.StatusChanged += gcw_StatusChanged;
                    gcw.PositionChanged += gcw_PositionChanged;
                    gcw.Start();

                    // Hook up event handler that runs after the asynchronous search completes.
                    cons.SearchCompleted += new EventHandler<ContactsSearchEventArgs>(Contacts_SearchCompleted);

                    //Start the asynchronous search.
                    cons.SearchAsync(String.Empty, FilterKind.None, "Contacts Test #1");
                };
        }

        void Contacts_SearchCompleted(object sender, ContactsSearchEventArgs e)
        {
            //Do something with the results.
            foreach (Contact c in e.Results)
            {
                var addresses = c.Addresses.ToList();

                if (addresses.Count > 0)               
                {
                    CivicAddress address = addresses[0].PhysicalAddress;
                    
                    if (address.StateProvince == "WA")
                    {
                        AddPushpin(c.DisplayName, address);
                    }
                }
            }
        }

        void AddPushpin(String name, CivicAddress address)
        {
            
            GeocodeServiceReference.GeocodeServiceClient gsc = new GeocodeServiceReference.GeocodeServiceClient("BasicHttpBinding_IGeocodeService");
            GeocodeRequest gRq = new GeocodeRequest();
            gRq.Credentials = new Microsoft.Phone.Controls.Maps.Credentials()
            {
                ApplicationId = App.Id,
            };
            gRq.Address = new Address()
            {
                AddressLine = address.AddressLine1,
                PostalTown = address.City,
                CountryRegion = address.CountryRegion,
            };

            gsc.GeocodeCompleted += new EventHandler<GeocodeCompletedEventArgs>((s, geocodeArgs) =>
            {
                GeoCoordinate addressLocation = geocodeArgs.Result.Results[0].Locations[0];
                Pushpin pin = new Pushpin();
                pin.Location = addressLocation;
                pin.Content = name;
                MyMap.Children.Add(pin);

                // Calculate the bounding rectangle containing the current center of the map, and the PushPin
                var viewRect = LocationRect.CreateLocationRect(new GeoCoordinate[] { viewModel.Center, addressLocation });
                // Resize the map to show that rectangle
                MyMap.SetView(viewRect);


            });
            gsc.GeocodeAsync(gRq);
        }


        void gcw_PositionChanged(object sender, GeoPositionChangedEventArgs<GeoCoordinate> e)
        {
            string latitude = e.Position.Location.Latitude.ToString();
            string longitude = e.Position.Location.Longitude.ToString();
            string Accuracy = e.Position.Location.HorizontalAccuracy.ToString();

            viewModel.Center = e.Position.Location;
        }

        void gcw_StatusChanged(object sender, GeoPositionStatusChangedEventArgs e)
        {
            string status = e.Status.ToString();
        }

        private void ButtonZoomIn_Click(object sender, RoutedEventArgs e)
        {
            viewModel.Zoom++;
        }

        private void ButtonZoomOut_Click(object sender, RoutedEventArgs e)
        {
            viewModel.Zoom--;
        }
    }
}