﻿using System;
using System.ComponentModel;
using Microsoft.Phone.Controls.Maps;
using System.Device.Location;
using System.Collections.ObjectModel;
using Microsoft.Phone.UserData;

namespace MapsEmulatorTest
{
    public class MapDataViewModel : INotifyPropertyChanged
    {
        private const double DefaultZoomLevel = 17.0;
        private const double MaxZoomLevel = 21.0;
        private const double MinZoomLevel = 1.0;
        private readonly CredentialsProvider _credentialsProvider = new ApplicationIdCredentialsProvider(App.Id);
        private GeoCoordinate _center;

        private double _zoom;
        public MapDataViewModel()
        {
            Zoom = DefaultZoomLevel;
        }

        public CredentialsProvider CredentialsProvider
        {
            get
            {
                return _credentialsProvider;
            }
        }

        public GeoCoordinate Center
        {
            get { return _center; }
            set
            {
                if (_center != value)
                {
                    _center = value;
                    NotifyPropertyChanged("Center");
                }
            }
        }

        public double Zoom
        {
            get { return _zoom; }
            set
            {
                var coercedZoom = Math.Max(MinZoomLevel, Math.Min(MaxZoomLevel, value));
                if (_zoom != coercedZoom)
                {
                    _zoom = value;
                    NotifyPropertyChanged("Zoom");
                }
            }
        }

        public event PropertyChangedEventHandler PropertyChanged = delegate { };

        private void NotifyPropertyChanged(string propertyName)
        {
            PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
