﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;

using Microsoft.Devices.Sensors;

namespace SilverCheeseRoller
{
    public partial class GamePage : PhoneApplicationPage
    {
        // For rendering the XAML onto a texture
        UIElementRenderer elementRenderer;

        ContentManager contentManager;
        GameTimer timer;
        SpriteBatch spriteBatch;

        Texture2D cheeseTexture;
        Microsoft.Xna.Framework.Rectangle cheeseDrawPosition;
        Vector2 cheesePosition;
        Vector2 cheeseSpeed;

        Accelerometer accel;

        float friction = 0.99f;

        public GamePage()
        {
            InitializeComponent();

            // Get the content manager from the application
            contentManager = (Application.Current as App).Content;

            // Create a timer for this page
            timer = new GameTimer();
            timer.UpdateInterval = TimeSpan.FromTicks(333333);
            timer.Update += OnUpdate;
            timer.Draw += OnDraw;

            accel = new Accelerometer();
            accel.CurrentValueChanged += new EventHandler<SensorReadingEventArgs<AccelerometerReading>>(accel_CurrentValueChanged);

            // Use the LayoutUpdate event to know when the page layout 
            // has completed so that we can create the UIElementRenderer.
            LayoutUpdated += new EventHandler(GamePage_LayoutUpdated);
        }

        void GamePage_LayoutUpdated(object sender, EventArgs e)
        {
            // Create the UIElementRenderer to draw the XAML page to a texture.

            // Check for 0 because when we navigate away the LayoutUpdate event
            // is raised but ActualWidth and ActualHeight will be 0 in that case.
            if ((ActualWidth > 0) && (ActualHeight > 0))
            {
                SharedGraphicsDeviceManager.Current.PreferredBackBufferWidth = (int)ActualWidth;
                SharedGraphicsDeviceManager.Current.PreferredBackBufferHeight = (int)ActualHeight;
            }

            if (null == elementRenderer)
            {
                elementRenderer = new UIElementRenderer(this, (int)ActualWidth, (int)ActualHeight);
            }
        }

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            // Set the sharing mode of the graphics device to turn on XNA rendering
            SharedGraphicsDeviceManager.Current.GraphicsDevice.SetSharingMode(true);

            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(SharedGraphicsDeviceManager.Current.GraphicsDevice);

            cheeseTexture = contentManager.Load<Texture2D>("Edam");

            // Create the draw rectangle 
            // Cheese should be a 9th the size of the screen
            cheeseDrawPosition = new Microsoft.Xna.Framework.Rectangle(
                0, 0, // top left hand corner of the screen
                (int)(this.ActualWidth / 9), // width
                (int)(this.ActualWidth / 9)  // height
                );

            accel.Start();

            // Start the timer
            timer.Start();

            base.OnNavigatedTo(e);
        }

        object accelLock = new object();

        Vector3 accelReading;

        void accel_CurrentValueChanged(object sender, SensorReadingEventArgs<AccelerometerReading> e)
        {
            lock (accelLock)
            {
                accelReading.X = (float)e.SensorReading.Acceleration.X;
                accelReading.Y = (float)e.SensorReading.Acceleration.Y;
                accelReading.Z = (float)e.SensorReading.Acceleration.Z;
            }
        }

        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            // Stop the timer
            timer.Stop();

            // Stop the accelerometer
            accel.Stop();

            // Set the sharing mode of the graphics device to turn off XNA rendering
            SharedGraphicsDeviceManager.Current.GraphicsDevice.SetSharingMode(false);

            base.OnNavigatedFrom(e);
        }

        /// <summary>
        /// Allows the page to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        private void OnUpdate(object sender, GameTimerEventArgs e)
        {
            cheesePosition = cheesePosition + cheeseSpeed;

            cheeseSpeed = cheeseSpeed * friction;

            lock (accelLock)
            {
                Vector2 cheeseAcceleration = new Vector2(accelReading.X, -accelReading.Y);
                cheeseSpeed = cheeseSpeed + (cheeseAcceleration / 5f);
            }

            cheeseDrawPosition.X = (int)(cheesePosition.X + 0.5f);
            cheeseDrawPosition.Y = (int)(cheesePosition.Y + 0.5f);

            if (cheeseDrawPosition.X < 0)
                cheeseSpeed.X = Math.Abs(cheeseSpeed.X);
            if (cheeseDrawPosition.Right > (int) this.ActualWidth)
                cheeseSpeed.X = -Math.Abs(cheeseSpeed.X);

            if (cheeseDrawPosition.Y < 0)
                cheeseSpeed.Y = Math.Abs(cheeseSpeed.Y);
            if (cheeseDrawPosition.Bottom > (int) this.ActualHeight)
                cheeseSpeed.Y = -Math.Abs(cheeseSpeed.Y);
        }

        /// <summary>
        /// Allows the page to draw itself.
        /// </summary>
        private void OnDraw(object sender, GameTimerEventArgs e)
        {
            // Render the Silverlight controls using the UIElementRenderer.
            elementRenderer.Render();

            SharedGraphicsDeviceManager.Current.GraphicsDevice.Clear(Color.CornflowerBlue);
            spriteBatch.Begin();

            // Using the texture from the UIElementRenderer, 
            // draw the Silverlight controls to the screen.
            spriteBatch.Draw(elementRenderer.Texture, Vector2.Zero, Color.White);

            spriteBatch.Draw(cheeseTexture, cheeseDrawPosition, Color.White);

            spriteBatch.End();
        }

        private void ResetButton_Click(object sender, RoutedEventArgs e)
        {
            cheesePosition.X = 0;
            cheesePosition.Y = 0;
        }

        private void quitButton_Click(object sender, RoutedEventArgs e)
        {
            NavigationService.Navigate(new Uri("/MainPage.xaml", UriKind.Relative));
        }
    }
}